Class: ActiveRecord::ConnectionAdapters::OracleEnhancedAdapter

Overview

Oracle enhanced adapter will work with both CRuby ruby-oci8 gem (which provides interface to Oracle OCI client) or with JRuby and Oracle JDBC driver.

It should work with Oracle 10g, 11g and 12c databases.

Usage notes:

  • Key generation assumes a “$table_name_seq” sequence is available for all tables; the sequence name can be changed using ActiveRecord::Base.set_sequence_name. When using Migrations, these sequences are created automatically. Use set_sequence_name :autogenerated with legacy tables that have triggers that populate primary keys automatically.

  • Oracle uses DATE or TIMESTAMP datatypes for both dates and times. Consequently some hacks are employed to map data back to Date or Time in Ruby. Timezones and sub-second precision on timestamps are not supported.

  • Default values that are functions (such as “SYSDATE”) are not supported. This is a restriction of the way ActiveRecord supports default values.

Required parameters:

  • :username

  • :password

  • :database - either TNS alias or connection string for OCI client or database name in JDBC connection string

Optional parameters:

  • :host - host name for JDBC connection, defaults to “localhost”

  • :port - port number for JDBC connection, defaults to 1521

  • :privilege - set “SYSDBA” if you want to connect with this privilege

  • :allow_concurrency - set to “true” if non-blocking mode should be enabled (just for OCI client)

  • :prefetch_rows - how many rows should be fetched at one time to increase performance, defaults to 100

  • :cursor_sharing - cursor sharing mode to minimize amount of unique statements, no default value

  • :time_zone - database session time zone (it is recommended to set it using ENV which will be then also used for database session time zone)

  • :schema - database schema which holds schema objects.

  • :tcp_keepalive - TCP keepalive is enabled for OCI client, defaults to true

  • :tcp_keepalive_time - TCP keepalive time for OCI client, defaults to 600

  • :jdbc_statement_cache_size - number of cached SQL cursors to keep open, disabled per default (for unpooled JDBC only)

  • :jdbc_connect_properties - Additional properties for establishing Oracle JDBC connection (for unpooled JDBC only) example to require encryption and checksumming for network connection:

    adapter: oracle_enhanced
    jdbc_connect_properties:
      'oracle.net.encryption_client': REQUIRED
      'oracle.net.crypto_checksum_client': REQUIRED
    

Optionals NLS parameters:

  • :nls_calendar

  • :nls_comp

  • :nls_currency

  • :nls_date_language

  • :nls_dual_currency

  • :nls_iso_currency

  • :nls_language

  • :nls_length_semantics - semantics of size of VARCHAR2 and CHAR columns, defaults to CHAR (meaning that size specifies number of characters and not bytes)

  • :nls_nchar_conv_excp

  • :nls_numeric_characters

  • :nls_sort

  • :nls_territory

  • :nls_timestamp_tz_format

  • :nls_time_format

  • :nls_time_tz_format

Fixed NLS values (not overridable):

  • :nls_date_format - format for :date columns is YYYY-MM-DD HH24:MI:SS

  • :nls_timestamp_format - format for :timestamp columns is YYYY-MM-DD HH24:MI:SS:FF6

Direct Known Subclasses

OracleAdapter

Defined Under Namespace

Classes: DatabaseTasks, StatementPool

Constant Summary collapse

ADAPTER_NAME =
"OracleEnhanced"
DEFAULT_NLS_PARAMETERS =

:stopdoc:

{
  nls_calendar: nil,
  nls_comp: nil,
  nls_currency: nil,
  nls_date_language: nil,
  nls_dual_currency: nil,
  nls_iso_currency: nil,
  nls_language: nil,
  nls_length_semantics: "CHAR",
  nls_nchar_conv_excp: nil,
  nls_numeric_characters: nil,
  nls_sort: nil,
  nls_territory: nil,
  nls_timestamp_tz_format: nil,
  nls_time_format: nil,
  nls_time_tz_format: nil
}
FIXED_NLS_PARAMETERS =

:stopdoc:

{
  nls_date_format: "YYYY-MM-DD HH24:MI:SS",
  nls_timestamp_format: "YYYY-MM-DD HH24:MI:SS:FF6"
}
NATIVE_DATABASE_TYPES =

:stopdoc:

{
  primary_key: "NUMBER(38) NOT NULL PRIMARY KEY",
  string: { name: "VARCHAR2", limit: 255 },
  text: { name: "CLOB" },
  ntext: { name: "NCLOB" },
  integer: { name: "NUMBER", limit: 38 },
  float: { name: "BINARY_FLOAT" },
  decimal: { name: "NUMBER" },
  datetime: { name: "TIMESTAMP" },
  timestamp: { name: "TIMESTAMP" },
  timestamptz: { name: "TIMESTAMP WITH TIME ZONE" },
  timestampltz: { name: "TIMESTAMP WITH LOCAL TIME ZONE" },
  time: { name: "TIMESTAMP" },
  date: { name: "DATE" },
  binary: { name: "BLOB" },
  boolean: { name: "NUMBER", limit: 1 },
  raw: { name: "RAW", limit: 2000 },
  bigint: { name: "NUMBER", limit: 19 }
}
NATIVE_DATABASE_TYPES_BOOLEAN_STRINGS =

if emulate_booleans_from_strings then store booleans in VARCHAR2

NATIVE_DATABASE_TYPES.dup.merge(
  boolean: { name: "VARCHAR2", limit: 1 }
)
AUTOGENERATED_SEQUENCE_NAME =

use in set_sequence_name to avoid fetching primary key value from sequence

"autogenerated"
VERSION =
File.read(File.expand_path("../../../../../VERSION", __FILE__)).chomp

Constants included from ActiveRecord::ConnectionAdapters::OracleEnhanced::StructureDump

ActiveRecord::ConnectionAdapters::OracleEnhanced::StructureDump::STATEMENT_TOKEN

Constants included from ActiveRecord::ConnectionAdapters::OracleEnhanced::DbmsOutput

ActiveRecord::ConnectionAdapters::OracleEnhanced::DbmsOutput::DBMS_OUTPUT_BUFFER_SIZE

Constants included from ActiveRecord::ConnectionAdapters::OracleEnhanced::DatabaseLimits

ActiveRecord::ConnectionAdapters::OracleEnhanced::DatabaseLimits::IDENTIFIER_MAX_LENGTH

Constants included from ActiveRecord::ConnectionAdapters::OracleEnhanced::Quoting

ActiveRecord::ConnectionAdapters::OracleEnhanced::Quoting::NONQUOTED_OBJECT_NAME, ActiveRecord::ConnectionAdapters::OracleEnhanced::Quoting::QUOTED_COLUMN_NAMES, ActiveRecord::ConnectionAdapters::OracleEnhanced::Quoting::QUOTED_TABLE_NAMES, ActiveRecord::ConnectionAdapters::OracleEnhanced::Quoting::VALID_TABLE_NAME

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Methods included from ActiveRecord::ConnectionAdapters::OracleEnhanced::StructureDump

#execute_structure_dump, #foreign_key_definition, #full_drop, #structure_drop, #structure_dump, #structure_dump_check_constraints, #structure_dump_column, #structure_dump_column_comments, #structure_dump_db_stored_code, #structure_dump_fk_constraints, #structure_dump_indexes, #structure_dump_primary_key, #structure_dump_synonyms, #structure_dump_table_comments, #structure_dump_unique_keys, #structure_dump_views, #structure_dump_virtual_column, #temp_table_drop

Methods included from ActiveRecord::ConnectionAdapters::OracleEnhanced::DbmsOutput

#dbms_output_enabled?, #disable_dbms_output, #enable_dbms_output

Methods included from ActiveRecord::ConnectionAdapters::OracleEnhanced::DatabaseLimits

#column_name_length, #in_clause_length, #sequence_name_length, #table_name_length

Methods included from ActiveRecord::ConnectionAdapters::OracleEnhanced::Quoting

mixed_case?, #quote, #quote_column_name_or_expression, #quote_string, #quoted_false, #quoted_true, #type_cast, #unquoted_false, #unquoted_true, valid_table_name?

Methods included from ActiveRecord::ConnectionAdapters::OracleEnhanced::OCIQuoting

#type_cast

Methods included from ActiveRecord::ConnectionAdapters::OracleEnhanced::JDBCQuoting

#type_cast

Methods included from ActiveRecord::ConnectionAdapters::OracleEnhanced::ContextIndex

#add_context_index, #remove_context_index

Methods included from ActiveRecord::ConnectionAdapters::OracleEnhanced::SchemaStatements

#add_column, #add_index, #add_index_options, #add_reference, #add_synonym, #add_timestamps, #aliased_types, #change_column, #change_column_comment, #change_column_default, #change_column_null, #change_table_comment, #column_comment, #columns, #create_alter_table, #create_schema_dumper, #create_table, #data_source_exists?, #data_sources, #disable_referential_integrity, #drop_table, #extract_foreign_key_action, #foreign_keys, #index_name, #index_name_exists?, #indexes, #insert_versions_sql, #materialized_views, #remove_column, #remove_columns, #remove_index, #remove_synonym, #rename_column, #rename_index, #rename_table, #synonyms, #table_comment, #table_exists?, #table_options, #tables, #tablespace, #type_to_sql, #update_table_definition, #views

Methods included from ActiveRecord::ConnectionAdapters::OracleEnhanced::DatabaseStatements

#affected_rows, #begin_db_transaction, #begin_isolated_db_transaction, #build_explain_clause, #cast_result, #commit_db_transaction, #create_savepoint, #default_sequence_name, #empty_insert_statement_value, #exec_insert, #exec_rollback_db_transaction, #exec_rollback_to_savepoint, #exec_update, #execute, #explain, #insert, #insert_fixture, #insert_fixtures_set, #raw_execute, #release_savepoint, #returning_column_values, #sql_for_insert, #supports_explain?, #transaction_isolation_levels, #write_lobs, #write_query?

Constructor Details

#initialize(config_or_deprecated_connection, deprecated_logger = nil, deprecated_connection_options = nil, deprecated_config = nil) ⇒ OracleEnhancedAdapter

:nodoc:



233
234
235
236
237
238
239
240
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 233

def initialize(config_or_deprecated_connection, deprecated_logger = nil, deprecated_connection_options = nil, deprecated_config = nil) # :nodoc:
  super(config_or_deprecated_connection, deprecated_logger, deprecated_connection_options, deprecated_config)

  @raw_connection = ConnectionAdapters::OracleEnhanced::Connection.create(@config)
  @enable_dbms_output = false
  @do_not_prefetch_primary_key = {}
  @columns_cache = {}
end

Instance Attribute Details

#auto_retryObject

If SQL statement fails due to lost connection then reconnect and retry SQL statement if autocommit mode is enabled. By default this functionality is disabled.



440
441
442
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 440

def auto_retry
  @auto_retry
end

Class Method Details

.clear_type_map!Object



741
742
743
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 741

def clear_type_map!
  @type_map = nil
end

.database_exists?(config) ⇒ Boolean

Oracle enhanced adapter has no implementation because Oracle Database cannot detect ‘NoDatabaseError`. Please refer to the following discussion for details. github.com/rsim/oracle-enhanced/pull/1900

Returns:

  • (Boolean)

Raises:

  • (NotImplementedError)


252
253
254
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 252

def self.database_exists?(config)
  raise NotImplementedError
end

.dbconsole(config, options = {}) ⇒ Object

Opens a database console session via sqlplus.

Called by Rails’ ‘bin/rails dbconsole` command on the adapter class returned from adapter registration. Builds an Oracle logon string of the form `user@database` and execs `sqlplus`.



274
275
276
277
278
279
280
281
282
283
284
285
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 274

def self.dbconsole(config, options = {})
  oracle_config = config.configuration_hash
  logon = +""

  if oracle_config[:username]
    logon << oracle_config[:username]
    logon << "/#{oracle_config[:password]}" if oracle_config[:password] && options[:include_password]
    logon << "@#{config.database}" if config.database
  end

  find_cmd_and_exec(ActiveRecord.database_cli[:oracle] || "sqlplus", logon)
end

.native_database_typesObject



732
733
734
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 732

def native_database_types
  emulate_booleans_from_strings ? NATIVE_DATABASE_TYPES_BOOLEAN_STRINGS : NATIVE_DATABASE_TYPES
end

.type_mapObject



736
737
738
739
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 736

def type_map
  @type_map ||= Type::TypeMap.new.tap { |m| initialize_type_map(m) }
  @type_map
end

Instance Method Details

#active?Boolean

Returns true if the connection is active.

Returns:

  • (Boolean)


455
456
457
458
459
460
461
462
463
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 455

def active? # :nodoc:
  # Pings the connection to check if it's still good. Note that an
  # #active? method is also available, but that simply returns the
  # last known state, which isn't good enough if the connection has
  # gone stale since the last use.
  _connection.ping
rescue OracleEnhanced::ConnectionException
  false
end

#adapter_nameObject

:nodoc:



244
245
246
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 244

def adapter_name # :nodoc:
  ADAPTER_NAME
end

#arel_visitorObject

:nodoc:



256
257
258
259
260
261
262
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 256

def arel_visitor # :nodoc:
  if supports_fetch_first_n_rows_and_offset?
    Arel::Visitors::Oracle12.new(self)
  else
    Arel::Visitors::Oracle.new(self)
  end
end

#bind_string(name, value) ⇒ Object

create bind object for type String



820
821
822
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 820

def bind_string(name, value)
  ActiveRecord::Relation::QueryAttribute.new(name, value, Type::OracleEnhanced::String.new)
end

#build_statement_poolObject



287
288
289
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 287

def build_statement_pool
  StatementPool.new(self.class.type_cast_config_to_integer(@config[:statement_limit]))
end

#check_versionObject



719
720
721
722
723
724
725
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 719

def check_version
  version = get_database_version.join(".").to_f

  if version < 10
    raise "Your version of Oracle (#{version}) is too old. Active Record Oracle enhanced adapter supports Oracle >= 10g."
  end
end

#clear_cache!(*args, **kwargs) ⇒ Object



479
480
481
482
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 479

def clear_cache!(*args, **kwargs)
  super
  self.class.clear_type_map!
end

#clear_table_columns_cache(table_name) ⇒ Object



616
617
618
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 616

def clear_table_columns_cache(table_name)
  @columns_cache[table_name.to_s] = nil
end

#column_definitions(table_name) ⇒ Object



590
591
592
593
594
595
596
597
598
599
600
601
602
603
604
605
606
607
608
609
610
611
612
613
614
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 590

def column_definitions(table_name)
  (owner, desc_table_name) = _connection.describe(table_name)

  select_all(<<~SQL.squish, "SCHEMA", [bind_string("owner", owner), bind_string("table_name", desc_table_name)])
    SELECT cols.column_name AS name, cols.data_type AS sql_type,
           cols.data_default, cols.nullable, cols.virtual_column, cols.hidden_column,
           cols.data_type_owner AS sql_type_owner,
           DECODE(cols.data_type, 'NUMBER', data_precision,
                             'FLOAT', data_precision,
                             'VARCHAR2', DECODE(char_used, 'C', char_length, data_length),
                             'RAW', DECODE(char_used, 'C', char_length, data_length),
                             'CHAR', DECODE(char_used, 'C', char_length, data_length),
                              NULL) AS limit,
           DECODE(data_type, 'NUMBER', data_scale, NULL) AS scale,
           comments.comments as column_comment
      FROM all_tab_cols cols, all_col_comments comments
     WHERE cols.owner      = :owner
       AND cols.table_name = :table_name
       AND cols.hidden_column = 'NO'
       AND cols.owner = comments.owner
       AND cols.table_name = comments.table_name
       AND cols.column_name = comments.column_name
     ORDER BY cols.column_id
  SQL
end

#columns_for_distinct(columns, orders) ⇒ Object

:nodoc:



680
681
682
683
684
685
686
687
688
689
690
691
692
693
694
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 680

def columns_for_distinct(columns, orders) # :nodoc:
  # construct a valid columns name for DISTINCT clause,
  # ie. one that includes the ORDER BY columns, using FIRST_VALUE such that
  # the inclusion of these columns doesn't invalidate the DISTINCT
  #
  # It does not construct DISTINCT clause. Just return column names for distinct.
  order_columns = orders.reject(&:blank?).map { |s|
      s = visitor.compile(s) unless s.is_a?(String)
      # remove any ASC/DESC modifiers
      s.gsub(/\s+(ASC|DESC)\s*?/i, "")
    }.reject(&:blank?).map.with_index { |column, i|
      "FIRST_VALUE(#{column}) OVER (PARTITION BY #{columns.join(', ')} ORDER BY #{column}) AS alias_#{i}__"
    }
  (order_columns << super).join(", ")
end

#current_databaseObject

Current database name



558
559
560
561
562
563
564
565
566
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 558

def current_database
  select_value(<<~SQL.squish, "SCHEMA")
    SELECT SYS_CONTEXT('userenv', 'con_name') FROM dual
  SQL
rescue ActiveRecord::StatementInvalid
  select_value(<<~SQL.squish, "SCHEMA")
    SELECT SYS_CONTEXT('userenv', 'db_name') FROM dual
  SQL
end

#current_schemaObject

Current database session schema



576
577
578
579
580
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 576

def current_schema
  select_value(<<~SQL.squish, "SCHEMA")
    SELECT SYS_CONTEXT('userenv', 'current_schema') FROM dual
  SQL
end

#current_userObject

Current database session user



569
570
571
572
573
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 569

def current_user
  select_value(<<~SQL.squish, "SCHEMA")
    SELECT SYS_CONTEXT('userenv', 'session_user') FROM dual
  SQL
end

#default_sequence_start_valueObject

:singleton-method: Specify default sequence start with value (by default 1 if not explicitly set), e.g.:

ActiveRecord::ConnectionAdapters::OracleEnhancedAdapter.default_sequence_start_value = 10000


196
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 196

cattr_accessor :default_sequence_start_value

#default_tablespaceObject

Default tablespace name of current user



583
584
585
586
587
588
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 583

def default_tablespace
  select_value(<<~SQL.squish, "SCHEMA")
    SELECT LOWER(default_tablespace) FROM user_users
    WHERE username = SYS_CONTEXT('userenv', 'current_schema')
  SQL
end

#default_tablespacesObject

:singleton-method: OracleEnhancedAdapter will use the default tablespace, but if you want specific types of objects to go into specific tablespaces, specify them like this in an initializer:

 ActiveRecord::ConnectionAdapters::OracleEnhancedAdapter.default_tablespaces =
{:clob => 'TS_LOB', :blob => 'TS_LOB', :index => 'TS_INDEX', :table => 'TS_DATA'}

Using the :tablespace option where available (e.g create_table) will take precedence over these settings.



168
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 168

cattr_accessor :default_tablespaces

#discard!Object



495
496
497
498
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 495

def discard!
  super
  _connection = nil
end

#disconnect!Object

Disconnects from the database.



490
491
492
493
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 490

def disconnect! # :nodoc:
  super
  _connection.logoff rescue nil
end

#emulate_booleansObject

:singleton-method: By default, the OracleEnhancedAdapter will consider all columns of type NUMBER(1) as boolean. If you wish to disable this emulation you can add the following line to your initializer file:

ActiveRecord::ConnectionAdapters::OracleEnhancedAdapter.emulate_booleans = false


155
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 155

cattr_accessor :emulate_booleans

#emulate_booleans_from_stringsObject

:singleton-method: If you wish that CHAR(1), VARCHAR2(1) columns are typecasted to booleans then you can add the following line to your initializer file:

ActiveRecord::ConnectionAdapters::OracleEnhancedAdapter.emulate_booleans_from_strings = true


177
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 177

cattr_accessor :emulate_booleans_from_strings

#extract_limit(sql_type) ⇒ Object

:nodoc:



791
792
793
794
795
796
797
798
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 791

def extract_limit(sql_type) # :nodoc:
  case sql_type
  when /^bigint/i
    19
  when /\((.*)\)/
    $1.to_i
  end
end

#extract_value_from_default(default) ⇒ Object



782
783
784
785
786
787
788
789
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 782

def extract_value_from_default(default)
  case default
  when String
    default.gsub("''", "'")
  else
    default
  end
end

#get_database_versionObject



715
716
717
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 715

def get_database_version
  _connection.database_version
end

#has_primary_key?(table_name, owner = nil, desc_table_name = nil) ⇒ Boolean

:nodoc:

Returns:

  • (Boolean)


660
661
662
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 660

def has_primary_key?(table_name, owner = nil, desc_table_name = nil) # :nodoc:
  !pk_and_sequence_for(table_name, owner, desc_table_name).nil?
end

#max_identifier_lengthObject Also known as: table_alias_length, index_name_length



703
704
705
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 703

def max_identifier_length
  supports_longer_identifier? ? 128 : 30
end

#max_index_name_sizeObject

This is to ensure rails is not shortening the index name, in order to preserve the local shortening behavior.



711
712
713
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 711

def max_index_name_size
  128
end

#native_database_typesObject

:startdoc:



430
431
432
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 430

def native_database_types # :nodoc:
  self.class.native_database_types
end

#next_sequence_value(sequence_name) ⇒ Object

Returns the next sequence value from a sequence generator. Not generally called directly; used by ActiveRecord to get the next primary key value when inserting a new database record (see #prefetch_primary_key?).



506
507
508
509
510
511
512
513
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 506

def next_sequence_value(sequence_name)
  # if sequence_name is set to :autogenerated then it means that primary key will be populated by trigger
  raise ArgumentError.new "Trigger based primary key is not supported" if sequence_name == AUTOGENERATED_SEQUENCE_NAME
  # call directly connection method to avoid prepared statement which causes fetching of next sequence value twice
  select_value(<<~SQL.squish, "SCHEMA")
    SELECT #{quote_table_name(sequence_name)}.NEXTVAL FROM dual
  SQL
end

#permissionsObject

:singleton-method: By default, OracleEnhanced adapter will grant unlimited tablespace, create session, create table, create view, and create sequence when running the rake task db:create.

If you wish to change these permissions you can add the following line to your initializer file:

ActiveRecord::ConnectionAdapters::OracleEnhancedAdapter.permissions =
["create session", "create table", "create view", "create sequence", "create trigger", "ctxapp"]


219
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 219

cattr_accessor :permissions

#pk_and_sequence_for(table_name, owner = nil, desc_table_name = nil) ⇒ Object

Find a table’s primary key and sequence. Note: Only primary key is implemented - sequence will be nil.



622
623
624
625
626
627
628
629
630
631
632
633
634
635
636
637
638
639
640
641
642
643
644
645
646
647
648
649
650
651
652
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 622

def pk_and_sequence_for(table_name, owner = nil, desc_table_name = nil) # :nodoc:
  (owner, desc_table_name) = _connection.describe(table_name)

  seqs = select_values_forcing_binds(<<~SQL.squish, "SCHEMA", [bind_string("owner", owner), bind_string("sequence_name", default_sequence_name(desc_table_name))])
    select us.sequence_name
    from all_sequences us
    where us.sequence_owner = :owner
    and us.sequence_name = upper(:sequence_name)
  SQL

  # changed back from user_constraints to all_constraints for consistency
  pks = select_values_forcing_binds(<<~SQL.squish, "SCHEMA", [bind_string("owner", owner), bind_string("table_name", desc_table_name)])
    SELECT cc.column_name
      FROM all_constraints c, all_cons_columns cc
     WHERE c.owner = :owner
       AND c.table_name = :table_name
       AND c.constraint_type = 'P'
       AND cc.owner = c.owner
       AND cc.constraint_name = c.constraint_name
  SQL

  warn <<~WARNING if pks.count > 1
    WARNING: Active Record does not support composite primary key.

    #{table_name} has composite primary key. Composite primary key is ignored.
  WARNING

  # only support single column keys
  pks.size == 1 ? [oracle_downcase(pks.first),
                   oracle_downcase(seqs.first)] : nil
end

#prefetch_primary_key?(table_name = nil) ⇒ Boolean

Returns true for Oracle adapter (since Oracle requires primary key values to be pre-fetched before insert). See also #next_sequence_value.

Returns:

  • (Boolean)


517
518
519
520
521
522
523
524
525
526
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 517

def prefetch_primary_key?(table_name = nil)
  return true if table_name.nil?
  table_name = table_name.to_s
  do_not_prefetch = @do_not_prefetch_primary_key[table_name]
  if do_not_prefetch.nil?
    owner, desc_table_name = _connection.describe(table_name)
    @do_not_prefetch_primary_key[table_name] = do_not_prefetch = !has_primary_key?(table_name, owner, desc_table_name)
  end
  !do_not_prefetch
end

#primary_key(table_name) ⇒ Object

Returns just a table’s primary key



655
656
657
658
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 655

def primary_key(table_name)
  pk_and_sequence = pk_and_sequence_for(table_name)
  pk_and_sequence && pk_and_sequence.first
end

#primary_keys(table_name) ⇒ Object

:nodoc:



664
665
666
667
668
669
670
671
672
673
674
675
676
677
678
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 664

def primary_keys(table_name) # :nodoc:
  (_owner, desc_table_name) = _connection.describe(table_name)

  pks = select_values_forcing_binds(<<~SQL.squish, "SCHEMA", [bind_string("table_name", desc_table_name)])
    SELECT cc.column_name
      FROM all_constraints c, all_cons_columns cc
     WHERE c.owner = SYS_CONTEXT('userenv', 'current_schema')
       AND c.table_name = :table_name
       AND c.constraint_type = 'P'
       AND cc.owner = c.owner
       AND cc.constraint_name = c.constraint_name
       order by cc.position
  SQL
  pks.map { |pk| oracle_downcase(pk) }
end

#raw_connectionObject

return raw OCI8 or JDBC connection



449
450
451
452
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 449

def raw_connection
  verify!
  _connection.raw_connection
end

#reconnectObject



465
466
467
468
469
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 465

def reconnect
  _connection.reset # tentative
rescue OracleEnhanced::ConnectionException
  connect
end

#reconnect!(restore_transactions: false) ⇒ Object

Reconnects to the database.



472
473
474
475
476
477
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 472

def reconnect!(restore_transactions: false) # :nodoc:
  super
  _connection.reset!
rescue OracleEnhanced::ConnectionException => e
  @logger.warn "#{adapter_name} automatic reconnection failed: #{e.message}" if @logger
end

#reset!Object



484
485
486
487
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 484

def reset!
  clear_cache!
  super
end

#reset_pk_sequence!(table_name, primary_key = nil, sequence_name = nil) ⇒ Object

:nodoc:



528
529
530
531
532
533
534
535
536
537
538
539
540
541
542
543
544
545
546
547
548
549
550
551
552
553
554
555
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 528

def reset_pk_sequence!(table_name, primary_key = nil, sequence_name = nil) # :nodoc:
  return nil unless data_source_exists?(table_name)
  unless primary_key && sequence_name
    # *Note*: Only primary key is implemented - sequence will be nil.
    primary_key, sequence_name = pk_and_sequence_for(table_name)
    # TODO This sequence_name implemantation is just enough
    # to satisty fixures. To get correct sequence_name always
    # pk_and_sequence_for method needs some work.
    begin
      sequence_name = table_name.classify.constantize.sequence_name
    rescue
      sequence_name = default_sequence_name(table_name)
    end
  end

  if @logger && primary_key && !sequence_name
    @logger.warn "#{table_name} has primary key #{primary_key} with no default sequence"
  end

  if primary_key && sequence_name
    new_start_value = select_value(<<~SQL.squish, "SCHEMA")
      select NVL(max(#{quote_column_name(primary_key)}),0) + 1 from #{quote_table_name(table_name)}
    SQL

    execute "DROP SEQUENCE #{quote_table_name(sequence_name)}"
    execute "CREATE SEQUENCE #{quote_table_name(sequence_name)} START WITH #{new_start_value}"
  end
end

#return_value_after_insert?(column) ⇒ Boolean

:nodoc:

Returns:

  • (Boolean)


264
265
266
267
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 264

def return_value_after_insert?(column) # :nodoc:
  # TODO: Return true if there this column will be populated (e.g by a sequence)
  super
end

#select_value_forcing_binds(arel, name, binds) ⇒ Object



836
837
838
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 836

def select_value_forcing_binds(arel, name, binds)
  single_value_from_rows(select_values_forcing_binds(arel, name, binds))
end

#select_values_forcing_binds(arel, name, binds) ⇒ Object

call select_values using binds even if surrounding SQL preparation/execution is done + # with conn.unprepared_statement (like AR.to_sql)



825
826
827
828
829
830
831
832
833
834
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 825

def select_values_forcing_binds(arel, name, binds)
  # remove possible force of unprepared SQL during dictionary access
  unprepared_statement_forced = prepared_statements_disabled_cache.include?(object_id)
  prepared_statements_disabled_cache.delete(object_id) if unprepared_statement_forced

  select_values(arel, name, binds)
ensure
  # Restore unprepared_statement setting for surrounding SQL
  prepared_statements_disabled_cache.add(object_id) if unprepared_statement_forced
end

#supports_comments?Boolean

Returns:

  • (Boolean)


328
329
330
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 328

def supports_comments?
  true
end

#supports_common_table_expressions?Boolean

Returns:

  • (Boolean)


307
308
309
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 307

def supports_common_table_expressions?
  true
end

#supports_datetime_with_precision?Boolean

Returns:

  • (Boolean)


324
325
326
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 324

def supports_datetime_with_precision?
  true
end

#supports_fetch_first_n_rows_and_offset?Boolean

Returns:

  • (Boolean)


315
316
317
318
319
320
321
322
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 315

def supports_fetch_first_n_rows_and_offset?
  false

  # TODO: At this point the connection is not initialized yet,
  # so `database_version` raises an error
  #
  # !use_old_oracle_visitor && database_version.first >= 12
end

#supports_foreign_keys?Boolean

Returns:

  • (Boolean)


299
300
301
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 299

def supports_foreign_keys?
  true
end

#supports_json?Boolean

Returns:

  • (Boolean)


340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 340

def supports_json?
  # Oracle Database 12.1 or higher version supports JSON.
  # However, Oracle enhanced adapter has limited support for JSON data type.
  # which does not pass many of ActiveRecord JSON tests.
  #
  # No migration supported for :json type due to there is no `JSON` data type
  # in Oracle Database itself.
  #
  # If you want to use JSON data type, here are steps
  # 1.Define :string or :text in migration
  #
  # create_table :test_posts, force: true do |t|
  #   t.string  :title
  #   t.text    :article
  # end
  #
  # 2. Set :json attributes
  #
  # class TestPost < ActiveRecord::Base
  #  attribute :title, :json
  #  attribute :article, :json
  # end
  #
  # 3. Add `is json` database constraints by running sql statements
  #
  # alter table test_posts add constraint test_posts_title_is_json check (title is json)
  # alter table test_posts add constraint test_posts_article_is_json check (article is json)
  #
  false
end

#supports_longer_identifier?Boolean

Returns:

  • (Boolean)


371
372
373
374
375
376
377
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 371

def supports_longer_identifier?
  if !use_shorter_identifier && database_version.to_s >= [12, 2].to_s
    true
  else
    false
  end
end

#supports_multi_insert?Boolean

Returns:

  • (Boolean)


332
333
334
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 332

def supports_multi_insert?
  database_version.to_s >= [11, 2].to_s
end

#supports_optimizer_hints?Boolean

Returns:

  • (Boolean)


303
304
305
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 303

def supports_optimizer_hints?
  true
end

#supports_savepoints?Boolean

:nodoc:

Returns:

  • (Boolean)


291
292
293
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 291

def supports_savepoints? # :nodoc:
  true
end

#supports_transaction_isolation?Boolean

:nodoc:

Returns:

  • (Boolean)


295
296
297
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 295

def supports_transaction_isolation? # :nodoc:
  true
end

#supports_views?Boolean

Returns:

  • (Boolean)


311
312
313
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 311

def supports_views?
  true
end

#supports_virtual_columns?Boolean

Returns:

  • (Boolean)


336
337
338
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 336

def supports_virtual_columns?
  database_version.first >= 11
end

#temporary_table?(table_name) ⇒ Boolean

:nodoc:

Returns:

  • (Boolean)


696
697
698
699
700
701
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 696

def temporary_table?(table_name) # :nodoc:
  select_value_forcing_binds(<<~SQL.squish, "SCHEMA", [bind_string("table_name", table_name.upcase)]) == "Y"
    SELECT
    temporary FROM all_tables WHERE table_name = :table_name and owner = SYS_CONTEXT('userenv', 'current_schema')
  SQL
end

#translate_exception(exception, message:, sql:, binds:) ⇒ Object

:nodoc:



800
801
802
803
804
805
806
807
808
809
810
811
812
813
814
815
816
817
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 800

def translate_exception(exception, message:, sql:, binds:) # :nodoc:
  case _connection.error_code(exception)
  when 1
    RecordNotUnique.new(message, sql: sql, binds: binds)
  when 60
    Deadlocked.new(message)
  when 900, 904, 942, 955, 1418, 2289, 2449, 17008
    ActiveRecord::StatementInvalid.new(message, sql: sql, binds: binds)
  when 1400
    ActiveRecord::NotNullViolation.new(message, sql: sql, binds: binds)
  when 2291, 2292
    InvalidForeignKey.new(message, sql: sql, binds: binds)
  when 12899
    ValueTooLong.new(message, sql: sql, binds: binds)
  else
    super
  end
end

#type_mapObject



778
779
780
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 778

def type_map
  self.class.type_map
end

#use_old_oracle_visitorObject

:singleton-method: By default, OracleEnhanced adapter will use Oracle12 visitor if database version is Oracle 12.1. If you wish to use Oracle visitor which is intended to work with Oracle 11.2 or lower for Oracle 12.1 database you can add the following line to your initializer file:

ActiveRecord::ConnectionAdapters::OracleEnhancedAdapter.use_old_oracle_visitor = true


188
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 188

cattr_accessor :use_old_oracle_visitor

#use_shorter_identifierObject

:singleton-method: By default, OracleEnhanced adapter will use longer 128 bytes identifier if database version is Oracle 12.2 or higher. If you wish to use shorter 30 byte identifier with Oracle Database supporting longer identifier you can add the following line to your initializer file:

ActiveRecord::ConnectionAdapters::OracleEnhancedAdapter.use_shorter_identifier = true


207
# File 'lib/active_record/connection_adapters/oracle_enhanced_adapter.rb', line 207

cattr_accessor :use_shorter_identifier